## Define variables for platform-specific behavior

MESSAGE("platform.cmake BEGIN")


## TODO: Accurate platform/target detection needed.  The below doesn't work correctly on all platforms.
if (WIN32)
	set (CoreARCH win32_x86)
	set (ARCH win32_x86)
endif ()

if (UNIX)
	set (CoreARCH linux_x86)
	set (ARCH linux_x86)
endif ()

include (${JavaGlue}/Platform/java.cmake)
#include (${CMAKE_SOURCE_DIR}/Platform/java.cmake)
if (APPLE)
	set (OSX true)
endif ()

if (OSX)
	set (CoreARCH osx_x86)
	set (ARCH osx_x86)
	set (SUB_OUT_DIR "")
endif ()

if(ANDROID)
# So these can be set via environment / -D
if ("${ANDROIDVER}" STREQUAL "")
   set (ANDROIDVER "$ENV{ANDROIDVER}")
endif ("${ANDROIDVER}" STREQUAL "")
if ("${ANDROIDVER}" STREQUAL "")
   set (ANDROIDVER "8")
endif ("${ANDROIDVER}" STREQUAL "")

  if("${ANDROIDVER}" STREQUAL "8")
    set(CMAKE_SYSTEM_NAME "android")
  else("${ANDROIDVER}" STREQUAL "8")
    set(CMAKE_SYSTEM_NAME "android-${ANDROIDVER}")
  endif("${ANDROIDVER}" STREQUAL "8")

#  set(CMAKE_TOOLCHAIN_FILE "${CoreTop}/Platform/${CMAKE_SYSTEM_NAME}/${CMAKE_SYSTEM_NAME}.toolchain")
  set(ANDROID_HOST ${CoreARCH})
endif(ANDROID)

if (ANDROID)
        set (CoreARCH ${CMAKE_SYSTEM_NAME}_arm)
        #set (CoreARCH android_arm)
	set (ARCH android_arm)
elseif (UNIX)
	    EXECUTE_PROCESS ( COMMAND "g++" "-dumpversion"
                OUTPUT_VARIABLE GPPVER
                ERROR_VARIABLE  GPPVER_ERROR
                RESULT_VARIABLE GPPVER_RESULT
		OUTPUT_STRIP_TRAILING_WHITESPACE)
	set (PLATFORM_INCLUDES /opt/local/include /usr/include/c++/${GPPVER} /usr/include/boost/tr1/tr1)
	set (PLATFORM_LIBS /opt/local/lib)
endif ()

if (MSVC)
	# It's the only way I've found to force /MT.
	# I tried all the methods explained online (e.g. using the ..._INIT macros), none of them worked.
	# set (CMAKE_CXX_FLAGS_DEBUG "/D_DEBUG /MTd /Zi /Ob0 /Od /RTC1")
	# set (CMAKE_CXX_FLAGS_MINSIZEREL "/MT /O1 /Ob1 /D NDEBUG")
	# set (CMAKE_CXX_FLAGS_RELEASE "/MT /O2 /Ob2 /D NDEBUG")
	# set (CMAKE_CXX_FLAGS_RELWITHDEBINFO "/MT /Zi /O2 /Ob1 /D NDEBUG")
	# (Also, I tried some methods of removing the MinSizeRel and RelWithDgbInfo configurations with SET(CMAKE_CONFIGURATION_TYPES ...), but, no dice.)

	set (CMAKE_CXX_FLAGS_DEBUG "${CMAKE_CXX_FLAGS_DEBUG} /DTRACE_DYNAMIC_MEMORY")

	add_definitions( "/D_CRT_SECURE_NO_WARNINGS" )
	add_definitions( "/DCURL_STATICLIB" )

elseif (OSX)
set (CMAKE_OSX_ARCHITECTURES "i386")
# set (CMAKE_OSX_ARCHITECTURES "i386;x86_64")
#	set (PLATFORM_C_FLAGS "-Wall -g -arch i386")
#	set (PLATFORM_LINK_FLAGS "-arch i386")
else ()
	# All other platforms
#	set (PLATFORM_C_FLAGS "-Wall -g")
endif ()

if (UNIX OR ANDROID)
 set (ZLIB z)
else ()
 set (ZLIB zlib)
endif ()



if (ANDROID)
  FOREACH(ANDROID_PATH_CANDIDATE
    $ENV{HOME}/opt/android $ENV{HOME}/opt/android-sdk-mac_86 $ENV{HOME}/opt/android-sdk-linux_86 $ENV{HOME}/opt/android-sdk-windows_86
    $ENV{HOME}/android $ENV{HOME}/android-sdk-mac_86 $ENV{HOME}/android-sdk-linux_86 $ENV{HOME}/android-sdk-windows_86
    /opt/android /opt/android-sdk-mac_86 /opt/android-sdk-linux_86 /opt/android-sdk-windows_86
    /usr/local/android /usr/local/android-sdk-mac_86 /usr/local/android-sdk-linux_86 /usr/local/android-sdk-windows_86
    /opt/local/android /opt/local/android-sdk-mac_86 /opt/local/android-sdk-linux_86 /opt/local/android-sdk-windows_86
    C:/opt/android C:/opt/android-sdk-mac_86 C:/opt/android-sdk-linux_86 C:/opt/android-sdk-windows_86
    C:/android C:/android-sdk-mac_86 C:/android-sdk-linux_86 C:/android-sdk-windows_86)
    IF( (("${ANDROID_HOME}" STREQUAL "") AND (EXISTS ${ANDROID_PATH_CANDIDATE})))
      SET(ANDROID_HOME "${ANDROID_PATH_CANDIDATE}")
    ENDIF( (("${ANDROID_HOME}" STREQUAL "") AND (EXISTS ${ANDROID_PATH_CANDIDATE})))
  ENDFOREACH(ANDROID_PATH_CANDIDATE)

  FOREACH(NDK_PATH_CANDIDATE
    $ENV{HOME}/opt/ndk $ENV{HOME}/opt/android-ndk-r4
    $ENV{HOME}/ndk $ENV{HOME}/android-ndk-r4
    /opt/ndk /opt/android-ndk-r4
    /usr/local/ndk /usr/local/android-ndk-r4
    /opt/local/ndk /opt/local/android-ndk-r4
    C:/opt/ndk C:/opt/android-ndk-r4
    C:/ndk C:/android-ndk-r4 )
    IF((("${NDK_HOME}" STREQUAL "") AND (EXISTS ${NDK_PATH_CANDIDATE})))
      SET(NDK_HOME "${NDK_PATH_CANDIDATE}")
    ENDIF((("${NDK_HOME}" STREQUAL "") AND (EXISTS ${NDK_PATH_CANDIDATE})))
  ENDFOREACH(NDK_PATH_CANDIDATE)
set (NDK_DIR ${NDK_HOME})


if(DEBUGCMAKE)
  message("ANDROIDVER: ${ANDROIDVER}")
endif(DEBUGCMAKE)

if ("${ANDROID_COMPILER}" STREQUAL "")
   set (ANDROID_COMPILER "$ENV{ANDROID_COMPILER}")
endif ("${ANDROID_COMPILER}" STREQUAL "")
if ("${ANDROID_COMPILER}" STREQUAL "")
   set (ANDROID_COMPILER "4.4.0")
endif ("${ANDROID_COMPILER}" STREQUAL "")

if (APPLE)
   # http://developer.apple.com/mac/library/qa/qa2001/qa1170.html
   # Find JAVA_HOME from /usr/libexec/java_home command
   set (ANDROID_HOST darwin-x86)
endif (APPLE)
if (LINUX)
   set (ANDROID_HOST linux-x86)
endif (LINUX)
# Needs checked
if (WIN32)
   set (ANDROID_HOST windows-x86)
endif (WIN32)

set (ANDROID_PLATFORM "${ANDROID_HOME}/platforms/android-${ANDROIDVER}")
MESSAGE("ANDROID_PLATFORM: ${ANDROID_PLATFORM}")

# Moved to {android,android-4}/{android,android-4}.toolchain
#ndk4:
#set (SYSROOT ${NDK_DIR}/build/platforms/android-${ANDROIDVER}/arch-arm)
#set (TOOLCHAIN "${NDK_DIR}/build/prebuild/${ANDROID_HOST}/arm-eabi-${ANDROID_COMPILER}")
#set (NDKBIN "${TOOLCHAIN}/arm-eabi/bin")
#set (NDKLIB "${TOOLCHAIN}/build/platforms/android-${ANDROIDVER}/arch-arm/usr/lib")

IF(DEBUGCMAKE)
MESSAGE ( "ANDROIDVER: ${ANDROIDVER}")
MESSAGE ( "ANDROID_HOME: ${ANDROID_HOST}")
if ("${JAVA_HOME}" STREQUAL "")
   set (JAVA_HOME "$ENV{JAVA_HOME}")
endif ("${JAVA_HOME}" STREQUAL "")
MESSAGE ( "JAVA_HOME: ${JAVA_HOME}")
MESSAGE ( "NDK_DIR: ${NDK_DIR}")
MESSAGE ( "ANDROID_PLATFORM: ${ANDROID_PLATFORM}")
MESSAGE ( "SYSROOT: ${SYSROOT}")
MESSAGE ( "TOOLCHAIN: ${TOOLCHAIN}")
MESSAGE ( "NDKBIN: ${NDKBIN}")
MESSAGE ( "NDKLIB: ${NDKLIB}")
ENDIF(DEBUGCMAKE)

endif (ANDROID)

# And path should have:
# export PATH=${PWD}:${ANDROID_HOME}/tools:${ANDROID_PLATFORM}/tools:${NDK_DIR}:$PATH

# MESSAGE ("platform.cmake END")
