/**
 * This file is public domain.  Make an application-specific copy and modify with types as needed.
 *
 * This file defines types that are added to the Javaglue inputs so that certain types are available for usage in interfaces.
 * Using this class, types that are allocated in C++ can be deleted cleanly from Java.
 * Do #define DELETE_CODE once to create code instance.
 *
 */

#ifndef _Included_xbig_basedelete_h
#define _Included_xbig_basedelete_h

#include <jni.h>
#include <string>
#include <vector>
#include <map>

namespace base {
typedef std::vector<unsigned char> ByteVector;
typedef std::vector<std::string> StringVector;
typedef std::vector<ByteVector> VectorByteVector;
typedef std::map<std::string, ByteVector> MapStringByteVector;
typedef std::map<long, ByteVector> MapLongByteVector;
typedef std::map<std::string, std::string> MapStringString;
}
class Delete {
  public:
    static void byteVector(base::ByteVector* bv);
    static void stringVector(base::StringVector* sv);
    static void vectorByteVector(base::VectorByteVector* vbv);
    static void mapStringByteVector(base::MapStringByteVector* msbv);
    static void mapLongByteVector(base::MapLongByteVector* mlbv);
    static void mapStringString(base::MapStringString* mss);
};

#ifdef DELETE_CODE
void Delete::byteVector(base::ByteVector* bv) { delete bv; }
void Delete::stringVector(base::StringVector* sv) { delete sv; }
void Delete::vectorByteVector(base::VectorByteVector* vbv) { delete vbv; }
void Delete::mapStringByteVector(base::MapStringByteVector* msbv) { delete msbv; }
void Delete::mapLongByteVector(base::MapLongByteVector* mlbv) { delete mlbv; }
void Delete::mapStringString(base::MapStringString* mss) { delete mss; }
#endif

// See jni_base.h for JNIEXPORTs
#endif // _Included_xbig_basedelete_h
